@@ -11,7 +11,6 @@
 This module contains converter classes and definitions.
 """
 from collections import namedtuple
-import string
 
 from .compat import ordered_dict_class
 from .exceptions import XMLSchemaTypeError, XMLSchemaValueError
@@ -38,9 +37,11 @@ class XMLSchemaConverter(NamespaceMapper):
     decoded XML data for an Element into a data structure and to build an Element
     from encoded data structure. There are two methods for interfacing the
     converter with the decoding/encoding process. The method *element_decode*
-    accepts ElementData instance, containing the element parts, and returns
-    a data structure. The method *element_encode* accepts a data structure
-    and returns an ElementData that can be
+    accepts an ElementData tuple, containing the element parts, and returns
+    a data structure. The method *element_encode* accepts a data structure and
+    returns an ElementData tuple. For default character data parts are ignored.
+    Prefixes and text key can be changed also using alphanumeric values but
+    ambiguities with schema elements could affect XML data re-encoding.
 
     :param namespaces: map from namespace prefixes to URI.
     :param dict_class: dictionary class to use for decoded data. Default is `dict`.
@@ -112,14 +113,9 @@ class XMLSchemaConverter(NamespaceMapper):
 
     def __setattr__(self, name, value):
         if name in {'attr_prefix', 'text_key', 'cdata_prefix'}:
-            if value is None:
-                pass
-            elif not isinstance(value, str):
+            if value is not None and not isinstance(value, str):
                 msg = '{} must be a str or None, not {}'
                 raise XMLSchemaTypeError(msg.format(name, type(value).__name__))
-            elif any(c in string.ascii_letters or c == '_' for c in value):
-                msg = '{} cannot includes letters or underscores: {!r}'
-                raise XMLSchemaValueError(msg.format(name, value))
 
             if name == 'attr_prefix':
                 self.ns_prefix = (value or '') + 'xmlns'
@@ -139,7 +135,7 @@ class XMLSchemaConverter(NamespaceMapper):
     @property
     def lossy(self):
         """The converter ignores some kind of XML data during decoding/encoding."""
-        return not self.cdata_prefix or not self.text_key or not self.attr_prefix
+        return self.cdata_prefix is None or self.text_key is None or self.attr_prefix is None
 
     @property
     def losslessly(self):
@@ -260,7 +256,7 @@ class XMLSchemaConverter(NamespaceMapper):
                 if v in schema_namespaces or v == XSI_NAMESPACE
             )
 
-        if xsd_type.is_simple() or xsd_type.has_simple_content():
+        if xsd_type.simple_type is not None:
             if data.attributes or self.force_dict and not xsd_type.is_simple():
                 result_dict.update(t for t in self.map_attributes(data.attributes))
                 if data.text is not None and data.text != '':
@@ -272,7 +268,7 @@ class XMLSchemaConverter(NamespaceMapper):
             if data.attributes:
                 result_dict.update(t for t in self.map_attributes(data.attributes))
 
-            has_single_group = xsd_type.content_type.is_single()
+            has_single_group = xsd_type.content.is_single()
             list_types = list if self.list is list else (self.list, list)
             if data.content:
                 for name, value, xsd_child in self.map_content(data.content):
@@ -322,7 +318,7 @@ class XMLSchemaConverter(NamespaceMapper):
                 pass
 
         if not isinstance(obj, (self.dict, dict)):
-            if xsd_element.type.is_simple() or xsd_element.type.has_simple_content():
+            if xsd_element.type.simple_type is not None:
                 return ElementData(tag, obj, None, {})
             elif xsd_element.type.mixed and not isinstance(obj, list):
                 return ElementData(tag, obj, None, {})
@@ -334,10 +330,11 @@ class XMLSchemaConverter(NamespaceMapper):
         attributes = {}
 
         for name, value in obj.items():
-            if name == self.text_key and self.text_key:
+            if name == self.text_key:
                 text = value
-            elif (self.cdata_prefix and name.startswith(self.cdata_prefix)) or \
-                    name[0].isdigit() and self.cdata_prefix == '':
+            elif self.cdata_prefix is not None and \
+                    name.startswith(self.cdata_prefix) and \
+                    name[len(self.cdata_prefix):].isdigit():
                 index = int(name[len(self.cdata_prefix):])
                 content.append((index, value))
             elif name == self.ns_prefix:
@@ -345,7 +342,9 @@ class XMLSchemaConverter(NamespaceMapper):
             elif name.startswith('%s:' % self.ns_prefix):
                 if not self.strip_namespaces:
                     self[name[len(self.ns_prefix) + 1:]] = value
-            elif self.attr_prefix and name.startswith(self.attr_prefix):
+            elif self.attr_prefix and \
+                    name.startswith(self.attr_prefix) and \
+                    name != self.attr_prefix:
                 attr_name = name[len(self.attr_prefix):]
                 ns_name = self.unmap_qname(attr_name, xsd_element.attributes)
                 attributes[ns_name] = value
@@ -356,7 +355,7 @@ class XMLSchemaConverter(NamespaceMapper):
                 content.extend((ns_name, item) for item in value)
             else:
                 ns_name = self.unmap_qname(name)
-                for xsd_child in xsd_element.type.content_type.iter_elements():
+                for xsd_child in xsd_element.type.content.iter_elements():
                     matched_element = xsd_child.match(ns_name, resolve=True)
                     if matched_element is not None:
                         if matched_element.type.is_list():
@@ -409,7 +408,7 @@ class UnorderedConverter(XMLSchemaConverter):
                 pass
 
         if not isinstance(obj, (self.dict, dict)):
-            if xsd_element.type.is_simple() or xsd_element.type.has_simple_content():
+            if xsd_element.type.simple_type is not None:
                 return ElementData(tag, obj, None, {})
             else:
                 return ElementData(tag, None, obj, {})
@@ -427,17 +426,20 @@ class UnorderedConverter(XMLSchemaConverter):
         content_lu = {}
 
         for name, value in obj.items():
-            if name == self.text_key and self.text_key:
+            if name == self.text_key:
                 text = value
-            elif (self.cdata_prefix and name.startswith(self.cdata_prefix)) or \
-                    name[0].isdigit() and self.cdata_prefix == '':
+            elif self.cdata_prefix is not None and \
+                    name.startswith(self.cdata_prefix) and \
+                    name[len(self.cdata_prefix):].isdigit():
                 index = int(name[len(self.cdata_prefix):])
                 content_lu[index] = value
             elif name == self.ns_prefix:
                 self[''] = value
             elif name.startswith('%s:' % self.ns_prefix):
                 self[name[len(self.ns_prefix) + 1:]] = value
-            elif self.attr_prefix and name.startswith(self.attr_prefix):
+            elif self.attr_prefix and \
+                    name.startswith(self.attr_prefix) and \
+                    name != self.attr_prefix:
                 attr_name = name[len(self.attr_prefix):]
                 ns_name = self.unmap_qname(attr_name, xsd_element.attributes)
                 attributes[ns_name] = value
@@ -448,7 +450,7 @@ class UnorderedConverter(XMLSchemaConverter):
             else:
                 # `value` is a list but not a list of lists or list of dicts.
                 ns_name = self.unmap_qname(name)
-                for xsd_child in xsd_element.type.content_type.iter_elements():
+                for xsd_child in xsd_element.type.content.iter_elements():
                     matched_element = xsd_child.match(ns_name, resolve=True)
                     if matched_element is not None:
                         if matched_element.type.is_list():
@@ -499,7 +501,7 @@ class ParkerConverter(XMLSchemaConverter):
     def element_decode(self, data, xsd_element, xsd_type=None, level=0):
         xsd_type = xsd_type or xsd_element.type
         preserve_root = self.preserve_root
-        if xsd_type.is_simple() or xsd_type.has_simple_content():
+        if xsd_type.simple_type is not None:
             if preserve_root:
                 return self.dict([(self.map_qname(data.tag), data.text)])
             else:
@@ -539,7 +541,7 @@ class ParkerConverter(XMLSchemaConverter):
         if not isinstance(obj, (self.dict, dict)):
             if obj == '':
                 obj = None
-            if xsd_element.type.is_simple() or xsd_element.type.has_simple_content():
+            if xsd_element.type.simple_type is not None:
                 return ElementData(xsd_element.name, obj, None, {})
             else:
                 return ElementData(xsd_element.name, None, obj, {})
@@ -564,7 +566,7 @@ class ParkerConverter(XMLSchemaConverter):
                         for item in value:
                             content.append((ns_name, item))
                     else:
-                        for xsd_child in xsd_element.type.content_type.iter_elements():
+                        for xsd_child in xsd_element.type.content.iter_elements():
                             matched_element = xsd_child.match(ns_name, resolve=True)
                             if matched_element is not None:
                                 if matched_element.type.is_list():
@@ -613,11 +615,11 @@ class BadgerFishConverter(XMLSchemaConverter):
         if has_local_root:
             result_dict['@xmlns'] = dict_class()
 
-        if xsd_type.is_simple() or xsd_type.has_simple_content():
+        if xsd_type.simple_type is not None:
             if data.text is not None and data.text != '':
-                result_dict[self.text_key] = data.text
+                result_dict['$'] = data.text
         else:
-            has_single_group = xsd_type.content_type.is_single()
+            has_single_group = xsd_type.content.is_single()
             list_types = list if self.list is list else (self.list, list)
             for name, value, xsd_child in self.map_content(data.content):
                 try:
@@ -682,14 +684,12 @@ class BadgerFishConverter(XMLSchemaConverter):
         for name, value in element_data.items():
             if name == '@xmlns':
                 continue
-            elif name == self.text_key and self.text_key:
-                text = element_data[self.text_key]
-            elif (self.cdata_prefix and name.startswith(self.cdata_prefix)) or \
-                    name[0].isdigit() and self.cdata_prefix == '':
-                index = int(name[len(self.cdata_prefix):])
-                content.append((index, value))
-            elif self.attr_prefix and name.startswith(self.attr_prefix):
-                attr_name = name[len(self.attr_prefix):]
+            elif name == '$':
+                text = value
+            elif name[0] == '$' and name[1:].isdigit():
+                content.append((int(name[1:]), value))
+            elif name[0] == '@':
+                attr_name = name[1:]
                 ns_name = self.unmap_qname(attr_name, xsd_element.attributes)
                 attributes[ns_name] = value
             elif not isinstance(value, (self.list, list)) or not value:
@@ -700,7 +700,7 @@ class BadgerFishConverter(XMLSchemaConverter):
                     content.append((ns_name, item))
             else:
                 ns_name = self.unmap_qname(name)
-                for xsd_child in xsd_element.type.content_type.iter_elements():
+                for xsd_child in xsd_element.type.content.iter_elements():
                     matched_element = xsd_child.match(ns_name, resolve=True)
                     if matched_element is not None:
                         if matched_element.type.is_list():
@@ -709,15 +709,7 @@ class BadgerFishConverter(XMLSchemaConverter):
                             content.extend((ns_name, item) for item in value)
                         break
                 else:
-                    if self.attr_prefix == '' and ns_name not in attributes:
-                        for xsd_attribute in xsd_element.attributes.values():
-                            if xsd_attribute.is_matching(ns_name):
-                                attributes[ns_name] = value
-                                break
-                        else:
-                            content.append((ns_name, value))
-                    else:
-                        content.append((ns_name, value))
+                    content.append((ns_name, value))
 
         return ElementData(tag, text, content, attributes)
 
@@ -746,7 +738,7 @@ class AbderaConverter(XMLSchemaConverter):
 
     def element_decode(self, data, xsd_element, xsd_type=None, level=0):
         xsd_type = xsd_type or xsd_element.type
-        if xsd_type.is_simple() or xsd_type.has_simple_content():
+        if xsd_type.simple_type is not None:
             children = data.text if data.text is not None and data.text != '' else None
         else:
             children = self.dict()
@@ -818,7 +810,7 @@ class AbderaConverter(XMLSchemaConverter):
                             content.append((ns_name, item))
                     else:
                         ns_name = self.unmap_qname(name)
-                        for xsd_child in xsd_element.type.content_type.iter_elements():
+                        for xsd_child in xsd_element.type.content.iter_elements():
                             matched_element = xsd_child.match(ns_name, resolve=True)
                             if matched_element is not None:
                                 if matched_element.type.is_list():
@@ -865,7 +857,7 @@ class JsonMLConverter(XMLSchemaConverter):
         if data.text is not None and data.text != '':
             result_list.append(data.text)
 
-        if not xsd_type.has_simple_content():
+        if xsd_type.model_group is not None:
             result_list.extend([
                 value if value is not None else self.list([name])
                 for name, value, _ in self.map_content(data.content)
@@ -910,8 +902,7 @@ class JsonMLConverter(XMLSchemaConverter):
 
         if data_len <= content_index:
             return ElementData(xsd_element.name, None, [], attributes)
-        elif data_len == content_index + 1 and \
-                (xsd_element.type.is_simple() or xsd_element.type.has_simple_content()):
+        elif data_len == content_index + 1 and xsd_element.type.simple_type is not None:
             return ElementData(xsd_element.name, obj[content_index], [], attributes)
         else:
             cdata_num = iter(range(1, data_len))
@@ -965,7 +956,7 @@ class ColumnarConverter(XMLSchemaConverter):
         else:
             result_dict = self.dict()
 
-        if xsd_type.is_simple() or xsd_type.has_simple_content():
+        if xsd_type.simple_type is not None:
             result_dict[xsd_element.local_name] = data.text or None
 
         if data.content:
@@ -978,15 +969,13 @@ class ColumnarConverter(XMLSchemaConverter):
                     name = name[2 + len(xsd_child.namespace):]
 
                 if xsd_child.is_single():
-                    if hasattr(xsd_child, 'type') and \
-                            (xsd_child.type.is_simple() or xsd_child.type.has_simple_content()):
+                    if hasattr(xsd_child, 'type') and xsd_child.type.simple_type is not None:
                         for k in value:
                             result_dict[k] = value[k]
                     else:
                         result_dict[name] = value
                 else:
-                    if hasattr(xsd_child, 'type') and \
-                            (xsd_child.type.is_simple() or xsd_child.type.has_simple_content()) \
+                    if hasattr(xsd_child, 'type') and xsd_child.type.simple_type is not None \
                             and not xsd_child.attributes:
                         if len(xsd_element.findall('*')) == 1:
                             try:
@@ -1024,7 +1013,7 @@ class ColumnarConverter(XMLSchemaConverter):
                 pass
 
         if not isinstance(obj, (self.dict, dict)):
-            if xsd_element.type.is_simple() or xsd_element.type.has_simple_content():
+            if xsd_element.type.simple_type is not None:
                 return ElementData(xsd_element.name, obj, None, {})
             elif xsd_element.type.mixed and not isinstance(obj, list):
                 return ElementData(xsd_element.name, obj, None, {})
@@ -1050,7 +1039,7 @@ class ColumnarConverter(XMLSchemaConverter):
                 content.extend((ns_name, item) for item in value)
             else:
                 ns_name = self.unmap_qname(name)
-                for xsd_child in xsd_element.type.content_type.iter_elements():
+                for xsd_child in xsd_element.type.content.iter_elements():
                     matched_element = xsd_child.match(ns_name, resolve=True)
                     if matched_element is not None:
                         if matched_element.type.is_list():
