@@ -7,6 +7,8 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
+import threading
+import elementpath
 from elementpath import XPath2Parser, XPathContext, ElementPathError
 from elementpath.datatypes import XSD_BUILTIN_TYPES
 
@@ -36,12 +38,26 @@ class XsdAssert(XsdComponent, ElementPathMixin):
 
     def __init__(self, elem, schema, parent, base_type):
         self.base_type = base_type
+        self._assert_xpath_lock = threading.Lock()  # Lock for assertion XPath operations
         super(XsdAssert, self).__init__(elem, schema, parent)
         ElementPathMixin.__init__(self)
 
     def __repr__(self):
         return '%s(test=%r)' % (self.__class__.__name__, self.path)
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        state.pop('_assert_xpath_lock', None)
+        state.pop('_xpath_lock', None)
+        state.pop('_xpath_parser', None)
+        state.pop('xpath_tokens', None)  # For schema objects
+        return state
+
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        self._xpath_lock = threading.Lock()
+        self._assert_xpath_lock = threading.Lock()
+
     def _parse(self):
         super(XsdAssert, self)._parse()
         if self.base_type.is_simple():
@@ -61,17 +77,23 @@ class XsdAssert(XsdComponent, ElementPathMixin):
     def built(self):
         return self.token is not None and (self.base_type.parent is None or self.base_type.built)
 
-    def parse_xpath_test(self):
-        # FIXME: parser's variables filled with XSD type with next elementpath minor release
+    def build(self):
         if not self.base_type.has_simple_content():
-            variables = {'value': XSD_BUILTIN_TYPES['anyType'].value}
+            builtin_type = XSD_BUILTIN_TYPES['anyType']
         else:
             try:
-                builtin_type_name = self.base_type.content_type.primitive_type.local_name
+                builtin_type_name = self.base_type.content.primitive_type.local_name
             except AttributeError:
-                variables = {'value': XSD_BUILTIN_TYPES['anySimpleType'].value}
+                builtin_type = XSD_BUILTIN_TYPES['anySimpleType']
             else:
-                variables = {'value': XSD_BUILTIN_TYPES[builtin_type_name].value}
+                builtin_type = XSD_BUILTIN_TYPES[builtin_type_name]
+
+        # Patch for compatibility with next elementpath minor release (v1.5)
+        # where parser variables will be filled with types.
+        if elementpath.__version__.startswith('1.4.'):
+            variables = {'value': builtin_type.value}
+        else:
+            variables = {'value': builtin_type}
 
         self.parser = XPath2Parser(
             namespaces=self.namespaces,
@@ -89,37 +111,33 @@ class XsdAssert(XsdComponent, ElementPathMixin):
         finally:
             self.parser.variables.clear()
 
-    def __call__(self, elem, value=None, source=None, namespaces=None, **kwargs):
-        if not self.parser.is_schema_bound():
-            self.parser.schema.bind_parser(self.parser)
+    def __call__(self, elem, value=None, source=None, **kwargs):
+        with self._xpath_lock:
+            if not self.parser.is_schema_bound():
+                self.parser.schema.bind_parser(self.parser)
 
         if value is not None:
             variables = {'value': self.base_type.text_decode(value)}
         else:
             variables = {'value': ''}
 
-        if source is None:
-            context = XPathContext(root=elem, variables=variables)
-        else:
+        if source is not None:
             context = XPathContext(root=source.root, item=elem, variables=variables)
-
-        default_namespace = self.parser.namespaces['']
-        if namespaces and '' in namespaces:
-            self.parser.namespaces[''] = namespaces['']
+        else:
+            # If validated from a component (could not work with rooted XPath expressions)
+            context = XPathContext(root=elem, variables=variables)
 
         try:
-            if not self.token.evaluate(context.copy()):
+            if not self.token.evaluate(context):
                 msg = "expression is not true with test path %r."
                 yield XMLSchemaValidationError(self, obj=elem, reason=msg % self.path)
         except ElementPathError as err:
             yield XMLSchemaValidationError(self, obj=elem, reason=str(err))
 
-        self.parser.namespaces[''] = default_namespace
-
     # For implementing ElementPathMixin
     def __iter__(self):
         if not self.parent.has_simple_content():
-            yield from self.parent.content_type.iter_elements()
+            yield from self.parent.content.iter_elements()
 
     @property
     def attrib(self):
