@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 #
-# Copyright (c), 2016-2019, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2020, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -8,17 +7,10 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-"""
-This module contains exception and warning classes for the 'xmlschema.validators' subpackage.
-"""
-from __future__ import unicode_literals
-
-from ..compat import PY3, string_base_type
 from ..exceptions import XMLSchemaException, XMLSchemaWarning, XMLSchemaValueError
 from ..qnames import qname_to_prefixed
 from ..etree import etree_tostring, etree_getpath
 from ..helpers import is_etree_element
-from ..resources import XMLResource
 
 
 class XMLSchemaValidatorError(XMLSchemaException):
@@ -35,7 +27,8 @@ class XMLSchemaValidatorError(XMLSchemaException):
     :type source: XMLResource
     :param namespaces: is an optional mapping from namespace prefix to URI.
     :type namespaces: dict
-    :ivar path: the XPath of the element, calculated when the element is set or the XML resource is set.
+    :ivar path: the XPath of the element, calculated when the element is set \
+    or the XML resource is set.
     """
     def __init__(self, validator, message, elem=None, source=None, namespaces=None):
         self.path = None
@@ -46,46 +39,36 @@ class XMLSchemaValidatorError(XMLSchemaException):
         self.elem = elem
 
     def __str__(self):
-        return unicode(self).encode("utf-8")
-
-    def __unicode__(self):
         if self.elem is None:
-            return '%s.' % self.message
-        else:
-            msg = ['%s:\n' % self.message]
-            if self.elem is not None:
-                elem_as_string = etree_tostring(self.elem, self.namespaces, '  ', 20)
-                if hasattr(self.elem, 'sourceline'):
-                    msg.append("Schema (line %r):\n\n%s\n" % (self.elem.sourceline, elem_as_string))
-                else:
-                    msg.append("Schema:\n\n%s\n" % elem_as_string)
-            if self.path is not None:
-                msg.append("Path: %s\n" % self.path)
-            if self.schema_url is not None:
-                msg.append("Schema URL: %s\n" % self.schema_url)
-                if self.origin_url not in (None, self.schema_url):
-                    msg.append("Origin URL: %s\n" % self.origin_url)
-            return '\n'.join(msg)
+            return self.message
 
-    if PY3:
-        __str__ = __unicode__
+        msg = ['%s:\n' % self.message]
+        elem_as_string = etree_tostring(self.elem, self.namespaces, '  ', 20)
+        msg.append("Schema:\n\n%s\n" % elem_as_string)
+
+        if self.path is not None:
+            msg.append("Path: %s\n" % self.path)
+        if self.schema_url is not None:
+            msg.append("Schema URL: %s\n" % self.schema_url)
+            if self.origin_url not in (None, self.schema_url):
+                msg.append("Origin URL: %s\n" % self.origin_url)
+        return '\n'.join(msg)
 
     @property
     def msg(self):
-        return self.__unicode__()
+        return self.__str__()
 
     def __setattr__(self, name, value):
         if name == 'elem' and value is not None:
             if not is_etree_element(value):
-                raise XMLSchemaValueError("'elem' attribute requires an Element, not %r." % type(value))
+                raise XMLSchemaValueError(
+                    "'elem' attribute requires an Element, not %r." % type(value)
+                )
             if self.source is not None:
-                self.path = etree_getpath(value, self.root, self.namespaces, relative=False, add_position=True)
+                self.path = etree_getpath(value, self.root, self.namespaces,
+                                          relative=False, add_position=True)
                 if self.source.is_lazy():
                     value = None  # Don't save the element of a lazy resource
-        if name == 'source' and value is not None and getattr(self, 'elem', None) is not None:
-            self.path = etree_getpath(self.elem, value.root, self.namespaces, relative=False, add_position=True)
-            if value.is_lazy():
-                self.elem = None
         super(XMLSchemaValidatorError, self).__setattr__(name, value)
 
     @property
@@ -176,7 +159,7 @@ class XMLSchemaModelError(XMLSchemaValidatorError, ValueError):
 class XMLSchemaModelDepthError(XMLSchemaModelError):
     """Raised when recursion depth is exceeded while iterating a model group."""
     def __init__(self, group):
-        msg = "maximum model recursion depth exceeded while iterating group %r" % group
+        msg = "maximum model recursion depth exceeded while iterating {!r}".format(group)
         super(XMLSchemaModelDepthError, self).__init__(group, message=msg)
 
 
@@ -199,7 +182,7 @@ class XMLSchemaValidationError(XMLSchemaValidatorError, ValueError):
     :type namespaces: dict
     """
     def __init__(self, validator, obj, reason=None, source=None, namespaces=None):
-        if not isinstance(obj, string_base_type):
+        if not isinstance(obj, str):
             _obj = obj
         else:
             _obj = obj.encode('ascii', 'xmlcharrefreplace').decode('utf-8')
@@ -214,11 +197,10 @@ class XMLSchemaValidationError(XMLSchemaValidatorError, ValueError):
         self.obj = obj
         self.reason = reason
 
-    def __str__(self):
-        # noinspection PyCompatibility,PyUnresolvedReferences
-        return unicode(self).encode("utf-8")
+    def __repr__(self):
+        return '%s(reason=%r)' % (self.__class__.__name__, self.reason)
 
-    def __unicode__(self):
+    def __str__(self):
         msg = ['%s:\n' % self.message]
         if self.reason is not None:
             msg.append('Reason: %s\n' % self.reason)
@@ -236,11 +218,10 @@ class XMLSchemaValidationError(XMLSchemaValidatorError, ValueError):
                 msg.append("Instance:\n\n%s\n" % elem_as_string)
         if self.path is not None:
             msg.append("Path: %s\n" % self.path)
+        if len(msg) == 1:
+            return msg[0][:-2]
         return '\n'.join(msg)
 
-    if PY3:
-        __str__ = __unicode__
-
 
 class XMLSchemaDecodeError(XMLSchemaValidationError):
     """
@@ -311,7 +292,8 @@ class XMLSchemaChildrenValidationError(XMLSchemaValidationError):
     :param namespaces: is an optional mapping from namespace prefix to URI.
     :type namespaces: dict
     """
-    def __init__(self, validator, elem, index, particle, occurs=0, expected=None, source=None, namespaces=None):
+    def __init__(self, validator, elem, index, particle, occurs=0,
+                 expected=None, source=None, namespaces=None):
         self.index = index
         self.particle = particle
         self.occurs = occurs
@@ -344,13 +326,14 @@ class XMLSchemaChildrenValidationError(XMLSchemaValidationError):
                     expected_tags.append('from %r namespace/s' % xsd_element.namespace)
 
             if not expected_tags:
-                pass  # reason += " No child element is expected at this point." <-- this can be misleading
+                pass
             elif len(expected_tags) == 1:
-                reason += " Tag %r expected." % expected_tags[0]
+                reason += " Tag %s expected." % expected_tags[0]
             else:
                 reason += " Tag (%s) expected." % ' | '.join(expected_tags)
 
-        super(XMLSchemaChildrenValidationError, self).__init__(validator, elem, reason, source, namespaces)
+        super(XMLSchemaChildrenValidationError, self).\
+            __init__(validator, elem, reason, source, namespaces)
 
 
 class XMLSchemaIncludeWarning(XMLSchemaWarning):
