@@ -36,8 +36,8 @@ from ..helpers import get_xsd_derivation_attribute, get_xsd_form_attribute
 from ..namespaces import XSD_NAMESPACE, XML_NAMESPACE, XSI_NAMESPACE, VC_NAMESPACE, \
     SCHEMAS_DIR, LOCATION_HINTS, NamespaceResourcesMap, NamespaceView, get_namespace
 from ..etree import etree_element, etree_tostring, prune_etree, ParseError
-from ..resources import is_remote_url, url_path_is_file, normalize_locations, \
-    fetch_resource, update_prefix, XMLResource
+from ..resources import is_local_url, is_remote_url, url_path_is_file, normalize_locations, \
+    fetch_resource, update_prefix, normalize_url, XMLResource
 from ..converters import XMLSchemaConverter
 from ..xpath import XMLSchemaProxy, ElementPathMixin
 
@@ -301,6 +301,10 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         elif build and global_maps is None:
             logger.setLevel(logging.WARNING)
 
+        if allow == 'sandbox' and base_url is None and is_local_url(source):
+            # Allow sandbox mode without a base_url using the initial schema URL as base
+            base_url = os.path.dirname(normalize_url(source))
+
         self.source = XMLResource(source, base_url, allow, defuse, timeout, lazy=False)
         logger.debug("Read schema from %r", self.source)
 
@@ -711,13 +715,13 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
 
         return group
 
-    def create_empty_content_group(self, parent, model='sequence'):
+    def create_empty_content_group(self, parent, model='sequence', **attrib):
         if model == 'sequence':
-            group_elem = etree_element(XSD_SEQUENCE)
+            group_elem = etree_element(XSD_SEQUENCE, **attrib)
         elif model == 'choice':
-            group_elem = etree_element(XSD_CHOICE)
+            group_elem = etree_element(XSD_CHOICE, **attrib)
         elif model == 'all':
-            group_elem = etree_element(XSD_ALL)
+            group_elem = etree_element(XSD_ALL, **attrib)
         else:
             raise XMLSchemaValueError("'model' argument must be (sequence | choice | all)")
 
@@ -756,8 +760,8 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
             parent=None,
             mixed=True
         )
-        any_type.content_type.append(self.BUILDERS.any_element_class(
-            ANY_ELEMENT, self, any_type.content_type
+        any_type.content.append(self.BUILDERS.any_element_class(
+            ANY_ELEMENT, self, any_type.content
         ))
         any_type.attributes[None] = self.BUILDERS.any_attribute_class(
             ANY_ATTRIBUTE_ELEMENT, self, any_type.attributes
@@ -903,17 +907,11 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         for xsd_global in self.iter_globals(self):
             yield from xsd_global.iter_components(xsd_classes)
 
-    def get_resource(self, source, **kwargs):
-        return XMLResource(
-            source=source,
-            base_url=kwargs.get('base_url'),
-            allow=kwargs.get('allow', self.allow),
-            defuse=kwargs.get('defuse', self.defuse),
-            timeout=kwargs.get('timeout', self.timeout),
-            lazy=kwargs.get('lazy', False)
-        )
-
     def get_schema(self, namespace):
+        """
+        Returns the first schema loaded for a namespace. Raises a
+        `KeyError` if the requested namespace is not loaded.
+        """
         try:
             return self.maps.namespaces[namespace][0]
         except KeyError:
@@ -1333,7 +1331,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         """
         self.check_validator(validation='lax')
         if not isinstance(source, XMLResource):
-            source = self.get_resource(source)
+            source = XMLResource(source, defuse=self.defuse, timeout=self.timeout, lazy=False)
         if not schema_path:
             schema_path = source.get_absolute_path(path)
 
@@ -1488,7 +1486,7 @@ class XMLSchemaBase(XsdValidator, ValidationMixin, ElementPathMixin, metaclass=X
         """
         self.check_validator(validation)
         if not isinstance(source, XMLResource):
-            source = self.get_resource(source, **kwargs)
+            source = XMLResource(source, defuse=self.defuse, timeout=self.timeout, lazy=False)
         if not schema_path and path:
             schema_path = source.get_absolute_path(path)
 
